﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab.Api.Types {
  public class MLDoubleArray : IMLValueVariable<double[,]> {
    private string[] _dataHeaders;

    #region Constructors

    public MLDoubleArray(string name, object values) {
      Name = name;
      if (!(values is double[,])) {
        throw new ArgumentException(string.Format("Invalid datatype for value: {0}", values.GetType()));
      }
      Data = values as double[,];

      _dataHeaders = new string[Data.GetLength(1)];
      for (int i = 0; i < _dataHeaders.Length; i++) {
        _dataHeaders[i] = string.Format("{0}:{1}", name, i);
      }
    }

    private MLDoubleArray(MLDoubleArray a1, MLDoubleArray a2) {
      Name = string.Format("{0};{1}", a1.Name, a2.Name);

      if (!ConcatFeasible(a1, a2)) {
        throw new ArgumentException(string.Format("Cannot concat {0} and {1}. Different number of rows.", a1.Name, a2.Name));
      }

      var numberOfColumns = a1.Columns + a2.Columns;
      _dataHeaders = new string[numberOfColumns];
      for (int i = 0; i < numberOfColumns; i++) {
        if (i < a1.Columns) {
          _dataHeaders[i] = a1.DataHeader[i];
        } else {
          _dataHeaders[i] = a2.DataHeader[i - a1.Columns];
        }
      }

      Data = new double[a1.Rows, numberOfColumns];
      for (int i = 0; i < a1.Rows; i++) {
        for (int j = 0; j < numberOfColumns; j++) {
          if (j < a1.Columns) {
            Data[i, j] = a1.Data[i, j];
          } else {
            Data[i, j] = a1.Data[i, j - a1.Columns];
          }
        }
      }
    }
    #endregion


    /// <summary>
    /// Concats two given MLDoubleArrays if their number of rows equals.
    /// If they can't be concated, this method returns null.
    /// </summary>
    /// <param name="a1"></param>
    /// <param name="a2"></param>
    /// <returns></returns>
    public static MLDoubleArray Concat(MLDoubleArray a1, MLDoubleArray a2) {
      if (ConcatFeasible(a1, a2)) {
        return null;
      }
      return new MLDoubleArray(a1, a2);
    }

    public static bool ConcatFeasible(MLDoubleArray a1, MLDoubleArray a2) {
      return a1.Rows == a2.Rows;
    }


    #region Properties from the interface

    public string Name { get; set; }

    public double[,] Data { get; set; }

    public MLDatatype Datatype {
      get { return MLDatatype.DoubleArray; }
    }

    public string[] DataHeader {
      get { return _dataHeaders; }
    }
    #endregion

    public int Rows { get {
        return Data.GetLength(0);
      }
    }

    public int Columns {
      get {
        return Data.GetLength(1);
      }
    }
  }


}
